__version__     = "0.1.5"

__title__       = "django-duo-auth"
__description__ = "A Django middleware that adds 2-factor authentication via Duo."
__uri__         = "https://github.com/Elemnir/django-duo-auth"

__author__      = "Adam Howard"
__email__       = "ahoward0920@gmail.com"

__license__     = "BSD 3-clause"
__copyright__   = "Copyright (c) 2019 Adam Howard"


