{% extends "base.html" %}

{% block head %}
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <meta http-equiv="X-UA-Compatible" content="IE=edge">
  <link rel="stylesheet" type="text/css" href="{{ duo_css_src }}">
{% endblock %}

{% block navbar %}
{% endblock %}

{% block content %}

{% if message %}
  <p>{{ message }}</p>
{% endif %}

<form method="POST" id="duo_form">
  {% csrf_token %}
  {% if next %}
    <input type="hidden" name="next" value="{{ next }}"/>
  {% endif %}
  {% if app_name %}
    <input type="hidden" name="app_name" value="{{ app_name }}"/>
  {% endif %}
</form>

<script src="{{ duo_js_src }}"></script>
<iframe id="duo_iframe"
        title="Two-Factor Authentication"
        frameborder="0"
        data-host="{{ duo_host }}"
        data-sig-request="{{ sig_request }}"
        data-post-action="{{ post_action }}"
        >
</iframe>

{% endblock %}

